require 'spec_helper'
require 'rspec/expectations'
#It is realtively writing specs in bin directory instead of lib directory
require 'puppet/../../bin/poolmembers.rb'

describe 'PoolMembers' do
  let(:env_members) { ["Env4Array01", "Env4Array02"] }
  let(:storage_response) { "Name       Status  Version    Disks Capacity   FreeSpace  Connections Pool   \n---------- ------- ---------- ----- ---------- ---------- ----------- -------\nEnv4Array0 online  V8.1.1 (R4 24    4.7TB      4.61TB     8           default \n  1                  17753)                                                   \nEnv4Array0 offline            0     0MB        0MB        0           Pool2   \n  2                                                                           \n" }
  let(:storage_response2) { "Name       Status  Version    Disks Capacity   FreeSpace  Connections Pool   \n---------- ------- ---------- ----- ---------- ---------- ----------- -------\nEnv4Array0 online  V8.1.1 (R4 24    4.7TB      4.61TB     8           default \n  1                  17753)                                                   \nEnv4Array0 offline            0     0MB        0MB        0           default   \n  2                                                                           \n" }
  let(:storage_response3) { "Name       Status  Version    Disks Capacity   FreeSpace  Connections Pool   \n---------- ------- ---------- ----- ---------- ---------- ----------- -------\nEnv4Array0 offline  V8.1.1 (R4 24    4.7TB      4.61TB     8           default \n  1                  17753)                                                   \nEnv4Array0  offline            0     0MB        0MB        0           Pool2   \n  2                                                                           \n" }
  let(:value) { {"Env4Array01" => {"pool" => "default", "status" => "online"},
                 "Env4Array02" => {"pool" => "Pool2", "status" => "offline"}} }
  let(:same_pool_with_different_status) { {"Env4Array01" => {"pool" => "default", "status" => "online"},
                                           "Env4Array02" => {"pool" => "default", "status" => "offline"}} }
  let(:offline_members) { {"Env4Array01" => {"pool" => "default", "status" => "offline"},
                           "Env4Array02" => {"pool" => "Pool2", "status" => "offline"}} }
  let(:storage_response_with_no_members) { "Name       Status  Version    Disks Capacity   FreeSpace  Connections Pool   \n---------- ------- ---------- ----- ---------- ---------- ----------- -------\n" }
  let(:resp) { "Env4Array01\nEnv4Array02\n" }

  describe "#get_pool_members" do
    it "Should parse existing members & respective  pool status into Hash structure" do
      expect(PoolMembers.get_pool_members(env_members, storage_response)).to eq(value)
    end

    it "Should parse members & return empty when no members exist" do
      expect(PoolMembers.get_pool_members(env_members, storage_response_with_no_members)).to eq({})
    end
  end

  describe "#get_storage_pool" do
    let(:server) { "172.17.4.10" }
    let(:uname) { "grpadmin" }
    let(:pwd) { "P@ssw0rd" }
    let(:member) { ["Env4Array01"] }
    let(:members) { ["Env4Array01", "Env4Array02"] }
    it "Should return the members which pool status were online" do
      PoolMembers.stub(:get_pool_info).and_return(storage_response)
      PoolMembers.stub(:get_pool_members).and_return(value)
      expect(PoolMembers.get_storage_pool(resp, server, uname, pwd)).to eq(member)
    end

    it "Should return different status of different members in same storage pool" do
      PoolMembers.stub(:get_pool_info).and_return(storage_response2)
      PoolMembers.stub(:get_pool_members).and_return(same_pool_with_different_status)
      expect(PoolMembers.get_storage_pool(resp, server, uname, pwd)).to eq(members)
    end

    it "Should return empty when no online member exist" do
      PoolMembers.stub(:get_pool_info).and_return(storage_response3)
      PoolMembers.stub(:get_pool_members).and_return(offline_members)
      expect(PoolMembers.get_storage_pool(resp, server, uname, pwd)).to eq([])
    end
  end
end
